###############################################################################
# THIS FILE CREATES DATA USING PARAMETERS DEFINED BY THE BLACK RAIL DATASET.  #
# A DESCRIPTION OF THE PARAMETERS AND DATA GENERATION FOLLOWS:                #
###############################################################################
if(0) {
	These datasets were created for the purposes of diagnosing program accuracy. 
	First, areas for two hundred sites were created
from a log normal distribution with mean and variance equal to the
mean and variance of the area of the sites in our network. Next, the
Universal Transverse Mercator locations (UTMs) for each site were
simulated from the mean and variance of the UTM Northing and Easting
for our study sites and interpatch distances were calculated. We set
the IFM parameters $\alpha$, $\beta$, $\delta$, $\mu$, and $\chi$
to 0.2, 0.5, 10, 0.5, and 0.25, respectively, and simulated dynamics
for 1000 years. The parameters were chosen such that the metapopulation
persisted with reasonable turnover. The last <XXX, user-specified>
years of these data were retained. For the detection data sets, we simulated
a removal design (\citealt{MacKenzie:2005wv}) based on three visits.
}

#-------------------
# IFM SIMULATE
#------------------
rm(list=ls())
set.seed(123)

#-------
#AREAS
#-------

mean.log.area=-0.75
sd.log.area=1.33

nsite=100
sim.area=rlnorm(nsite,mean.log.area,sd.log.area)
log.sim.area=log(sim.area+1)

#---------
#DISTANCE
#--------

sim.site.x=rnorm(nsite,643930,9000)
sim.site.y=rnorm(nsite,4340500,10500)

sim.site.x.mat.col=matrix(rep(sim.site.x,nsite),ncol=nsite,byrow=TRUE)
sim.site.x.mat.row=matrix(rep(sim.site.x,nsite),ncol=nsite)
sim.site.delta.x=(sim.site.x.mat.col-sim.site.x.mat.row)^2

sim.site.y.mat.col=matrix(rep(sim.site.y,nsite),ncol=nsite,byrow=TRUE)
sim.site.y.mat.row=matrix(rep(sim.site.y,nsite),ncol=nsite)
sim.site.delta.y=(sim.site.y.mat.col-sim.site.y.mat.row)^2

sim.distance=((sim.site.delta.x+sim.site.delta.y)^0.5)/1000
diag(sim.distance)=99

#-------------#
# CREATE SPOM #
#-------------#

spom=function(nsite,nyear.sim,alpha,beta,gamma,mu,chi) {
	psi=matrix(rep(NA,nsite*nyear.sim),ncol=nyear.sim)
	psi1=rbinom(nsite,1,0.8)
	psi[,1]=psi1

	s.i.temp=exp(-alpha*sim.distance)
	s.i.temp[s.i.temp==1]=0

	e.i=mu/sim.area^chi
	e.i[e.i>1]=1
	
	for (i in 2:nyear.sim) {
		s.i=s.i.temp%*%((sim.area^beta)*psi[,i-1])
		c.i=s.i^2/(s.i^2+gamma^2) 
		e.i.re=e.i*(1-c.i)	

		mu1=psi[,i-1]*(1-e.i.re)+(1-psi[,i-1])*c.i
		psi.temp=rbinom(nsite,1,mu1)
		psi[,i]=psi.temp
		}
	psi
	}

#---------------
#IFM PARAMETERS
#---------------
nyear.sim=1000

set.seed(12345)
psi.sim=spom(nsite,nyear.sim,alpha=0.2,beta=0.5,gamma=10,mu=0.25,chi=0.25)

nyear=20
z.sim=psi.sim[,(nyear.sim+1-nyear):nyear.sim]
apply(z.sim,2,mean)

#CREATE DET HISTORY
p=rep(c(0.4,0.6,0.2,0.5,0.3),4)
nrep=5
temp=rep(1,nrep*nsite*nyear)
p.mat=matrix(rep(p,nsite),nrow=nsite,byrow=TRUE)
temp.z.sim=z.sim*p.mat
sim.det=rbinom(temp,1,temp.z.sim)
dim(sim.det)=c(nsite,nyear,nrep)
z.sim.no.missing.values=z.sim

#ENFORCE REMOVAL DESIGN
for (i in 1:nsite) {
  for(t in 1:nyear) {
    if (sim.det[i,t,1]==1) sim.det[i,t,2]=2
    if (sim.det[i,t,1]==1) sim.det[i,t,3]=2
    if (sim.det[i,t,2]==1) sim.det[i,t,3]=2
  }
}
sim.det[sim.det==2]=NA
sim.det.no.missing.values=sim.det


#RANDOMLY CREATE MISSING DATA
set.seed(123)
unif.mat=runif(nyear*nsite)
z.sim.05=z.sim
z.sim.05[unif.mat<0.05]=NA
z.sim.30=z.sim
z.sim.30[unif.mat<0.3]=NA

sim.det.05=sim.det
sim.det.05[rep(is.na(z.sim.05),nrep)]=NA

sim.det.30=sim.det
sim.det.30[rep(is.na(z.sim.30),nrep)]=NA

save(z.sim.no.missing.values,z.sim.05,z.sim.30,sim.det.no.missing.values,sim.det.05,sim.det.30,sim.area,sim.distance,file=paste("SIMULATE_DATA_",nsite,"_",nyear,"_",nrep,".RData",sep=""))
